@@ -25,6 +25,15 @@
 
 /*! \addtogroup osmux Osmocom Multiplex Protocol
  *  @{
+ *
+ *  This code implements a variety of utility functions related to the
+ *  OSMUX user-plane multiplexing protocol, an efficient alternative to
+ *  plain UDP/RTP streams for voice transport in back-haul of cellular
+ *  networks.
+ *
+ *  For information about the OSMUX protocol design, please see the
+ *  OSMUX reference manual at
+ *  http://ftp.osmocom.org/docs/latest/osmux-reference.pdf
  */
 
 /*! \file osmux.c
@@ -846,19 +855,20 @@ void osmux_xfrm_output_init(struct osmux_out_handle *h, uint32_t rtp_ssrc)
 	h->rtp_ssrc = rtp_ssrc;
 }
 
-#define SNPRINTF_BUFFER_SIZE(ret, size, len, offset)	\
-	size -= ret;					\
-	if (ret > len)					\
-		ret = len;				\
+#define SNPRINTF_BUFFER_SIZE(ret, remain, offset)	\
+	if (ret < 0)					\
+		ret = 0;				\
 	offset += ret;					\
-	len -= ret;
+	if (ret > remain)				\
+		ret = remain;				\
+	remain -= ret;
 
 static int osmux_snprintf_header(char *buf, size_t size, struct osmux_hdr *osmuxh)
 {
+	unsigned int remain = size, offset = 0;
 	int ret;
-	int len = size, offset = 0;
 
-	ret = snprintf(buf, len, "OSMUX seq=%03u ccid=%03u "
+	ret = snprintf(buf, remain, "OSMUX seq=%03u ccid=%03u "
 				 "ft=%01u ctr=%01u "
 				 "amr_f=%01u amr_q=%01u "
 				 "amr_ft=%02u amr_cmr=%02u ",
@@ -866,7 +876,7 @@ static int osmux_snprintf_header(char *buf, size_t size, struct osmux_hdr *osmux
 			osmuxh->ft, osmuxh->ctr,
 			osmuxh->amr_f, osmuxh->amr_q,
 			osmuxh->amr_ft, osmuxh->amr_cmr);
-	SNPRINTF_BUFFER_SIZE(ret, size, len, offset);
+	SNPRINTF_BUFFER_SIZE(ret, remain, offset);
 
 	return offset;
 }
@@ -874,31 +884,45 @@ static int osmux_snprintf_header(char *buf, size_t size, struct osmux_hdr *osmux
 static int osmux_snprintf_payload(char *buf, size_t size,
 				  const uint8_t *payload, int payload_len)
 {
+	unsigned int remain = size, offset = 0;
 	int ret, i;
-	int len = size, offset = 0;
 
-	ret = snprintf(buf+offset, len, "[ ");
-	SNPRINTF_BUFFER_SIZE(ret, size, len, offset);
+	ret = snprintf(buf + offset, remain, "[ ");
+	SNPRINTF_BUFFER_SIZE(ret, remain, offset);
 
 	for (i=0; i<payload_len; i++) {
-		ret = snprintf(buf+offset, len, "%02x ", payload[i]);
-		SNPRINTF_BUFFER_SIZE(ret, size, len, offset);
+		ret = snprintf(buf + offset, remain, "%02x ", payload[i]);
+		SNPRINTF_BUFFER_SIZE(ret, remain, offset);
 	}
 
-	ret = snprintf(buf+offset, len, "] ");
-	SNPRINTF_BUFFER_SIZE(ret, size, len, offset);
+	ret = snprintf(buf + offset, remain, "]");
+	SNPRINTF_BUFFER_SIZE(ret, remain, offset);
 
 	return offset;
 }
 
-
+/*! Print osmux header fields and payload from msg into buffer buf.
+ *  \param[out] buf buffer to store the output into
+ *  \param[in] len length of buf in bytes
+ *  \param[in] msgb message buffer containing one or more osmux frames
+ *  \returns the number of characters printed (excluding the null byte used to end output to strings).
+ *
+ * If the output was truncated due to this limit, then the return value is the number of characters
+ * (excluding the terminating null byte) which would have been written to the final string if enough
+ * space had been available.
+ */
 int osmux_snprintf(char *buf, size_t size, struct msgb *msg)
 {
-	int ret;
-	unsigned int offset = 0;
-	int msg_len = msg->len, len = size;
+	unsigned int remain = size;
+	unsigned int msg_off = 0;
 	struct osmux_hdr *osmuxh;
-	int this_len, msg_off = 0;
+	unsigned int offset = 0;
+	int msg_len = msg->len;
+	uint32_t payload_len;
+	int ret;
+
+	if (size)
+		buf[0] = '\0';
 
 	while (msg_len > 0) {
 		if (msg_len < sizeof(struct osmux_hdr)) {
@@ -909,38 +933,50 @@ int osmux_snprintf(char *buf, size_t size, struct msgb *msg)
 		}
 		osmuxh = (struct osmux_hdr *)((uint8_t *)msg->data + msg_off);
 
-		if (!osmo_amr_ft_valid(osmuxh->amr_ft)) {
-			LOGP(DLMIB, LOGL_ERROR, "Bad AMR FT %d, skipping\n",
-			     osmuxh->amr_ft);
+		ret = osmux_snprintf_header(buf + offset, remain, osmuxh);
+		SNPRINTF_BUFFER_SIZE(ret, remain, offset);
+
+		msg_off += sizeof(struct osmux_hdr);
+		msg_len -= sizeof(struct osmux_hdr);
+
+		switch (osmuxh->ft) {
+		case OSMUX_FT_SIGNAL:
+			ret = snprintf(buf + offset, remain, "[signal]");
+			SNPRINTF_BUFFER_SIZE(ret, remain, offset);
+			return -1;
+		case OSMUX_FT_DUMMY:
+		case OSMUX_FT_VOICE_AMR:
+			if (!osmo_amr_ft_valid(osmuxh->amr_ft)) {
+				LOGP(DLMIB, LOGL_ERROR, "Bad AMR FT %d, skipping\n",
+				     osmuxh->amr_ft);
+				return -1;
+			}
+
+			payload_len = osmux_get_payload_len(osmuxh);
+
+			if (msg_len < payload_len) {
+				LOGP(DLMIB, LOGL_ERROR,
+				     "No room for OSMUX payload: only %d bytes\n",
+				     msg_len);
+				return -1;
+			}
+
+			if (osmuxh->ft == OSMUX_FT_VOICE_AMR) {
+				ret = osmux_snprintf_payload(buf + offset, remain,
+							     osmux_get_payload(osmuxh),
+							     payload_len);
+				SNPRINTF_BUFFER_SIZE(ret, remain, offset);
+			}
+
+			msg_off += payload_len;
+			msg_len -= payload_len;
+			break;
+		default:
+			LOGP(DLMIB, LOGL_ERROR, "Unknown OSMUX ft value %d\n",
+			     osmuxh->ft);
 			return -1;
 		}
-
-		ret = osmux_snprintf_header(buf+offset, size, osmuxh);
-		if (ret < 0)
-			break;
-		SNPRINTF_BUFFER_SIZE(ret, size, len, offset);
-
-		this_len = sizeof(struct osmux_hdr) +
-			   osmux_get_payload_len(osmuxh);
-		msg_off += this_len;
-
-		if (msg_len < this_len) {
-			LOGP(DLMIB, LOGL_ERROR,
-			     "No room for OSMUX payload: only %d bytes\n",
-			     msg_len);
-			return -1;
-		}
-
-		ret = osmux_snprintf_payload(buf+offset, size,
-					     osmux_get_payload(osmuxh),
-					     osmux_get_payload_len(osmuxh));
-		if (ret < 0)
-			break;
-		SNPRINTF_BUFFER_SIZE(ret, size, len, offset);
-
-		msg_len -= this_len;
 	}
-
 	return offset;
 }
 
